/**
 * Copyright (c) 2006 - 2012 Smaxe Ltd (www.smaxe.com).
 * All rights reserved.
 */
import com.smaxe.uv.media.core.IMedia;
import com.smaxe.uv.media.core.IMediaFileReader;
import com.smaxe.uv.media.core.MediaTrackInfo;
import com.smaxe.uv.media.core.VideoTrackInfo;
import com.smaxe.uv.media.sdk.IMediaCodecSdk;
import com.smaxe.uv.media.sdk.IMediaContainerSdk;
import com.smaxe.uv.media.sdk.IMediaSdk;
import com.smaxe.uv.media.sdk.MediaSdkFactory;
import com.smaxe.uv.media.sdk.IMediaCodecSdk.ISnapshotFactory;
import com.smaxe.uv.media.util.IterableIterator;

import java.io.File;
import java.io.FileOutputStream;
import java.util.Iterator;

/**
 * <code>ExMediaSdkMediaCodec</code> - {@link IMediaSdk} example to decode media files. 
 * 
 * @author Andrei Sochirca
 * @see <a href="http://www.smaxe.com/product.jsf?id=juv-media-sdk" target="_blank">JUV Media SDK</a>
 */
public final class ExMediaSdkMediaCodec extends Object
{
    /**
     * Entry point.
     * 
     * @param args
     * @throws Exception
     */
    public static void main(final String[] args) throws Exception
    {
        // NOTE:
        // Evaluation version has some limitations:
        // - video decoder watermarks frames (and blacks it sometimes)
        
        
        IMediaSdk mediaSdk = MediaSdkFactory.getMediaSdk();
        
        // 
        createThumbnailFromFirstVideoFrame(mediaSdk, new File("sample.flv"));
    }
    
    /**
     * Creates thumbnail (size: 160x120) from the first video frame.
     * 
     * @param mediaSdk
     * @param file media file
     * @throws Exception if an exception occurred
     */
    public static void createThumbnailFromFirstVideoFrame(final IMediaSdk mediaSdk, final File file) throws Exception
    {
        final IMediaContainerSdk containerSdk = mediaSdk.getMediaContainerSdk();
        final IMediaCodecSdk codecSdk = mediaSdk.getMediaCodecSdk();
        
        final IMediaFileReader reader = containerSdk.openFileReader(file);
        
        final Iterator<IMedia.ITrackSample> samples = reader.getTrackSamples(MediaTrackInfo.ALL_TRACKS, 0 /*timestamp*/);
        
        for (IMedia.ITrackSample sample : new IterableIterator<IMedia.ITrackSample>(samples))
        {
            final MediaTrackInfo track = sample.getTrack();
            if (!(track.type == MediaTrackInfo.VIDEO && sample.isKey())) continue;
            
            // 'new VideoTrackInfo(160 /*width*/, 120 /*height*/)' defines thumbnail size,
            // if not set then video dimensions are used 
            ISnapshotFactory snapshotFactory = codecSdk.createSnapshotFactory(track.codec, track.codecConfig,
                    new VideoTrackInfo(160 /*width*/, 120 /*height*/));
            
            FileOutputStream fos = new FileOutputStream(file);
            
            try
            {
                snapshotFactory.generateAsJpg(fos, reader.readTrackSample(sample, new byte[(int) sample.getSize()], 0));
            }
            finally
            {
                fos.close();
            }
            
            break;
        }
        
        reader.close();
    }
    
    /**
     * Constructor.
     */
    private ExMediaSdkMediaCodec()
    {
    }
}