/**
 * Copyright (c) 2008 - 2015 Smaxe Ltd (www.smaxe.com).
 * All rights reserved.
 */

import com.smaxe.io.ByteArray;
import com.smaxe.logger.ILogger;
import com.smaxe.logger.support.Loggers;
import com.smaxe.uv.client.rtmp.MediaDataFactory;
import com.smaxe.uv.client.rtmp.video.FlvVideo;
import com.smaxe.uv.client.x.RtmpPlayer;
import com.smaxe.uv.media.core.IVideoDecoder;
import com.smaxe.uv.media.core.MediaTrackInfo;
import com.smaxe.uv.media.core.VideoFrame;
import com.smaxe.uv.media.java.swing.JVideoScreen;
import com.smaxe.uv.media.sdk.IMediaSdk;
import com.smaxe.uv.media.sdk.MediaSdkFactory;
import com.smaxe.uv.stream.IMediaStreamAcceptor;
import com.smaxe.uv.stream.MediaData;

import java.awt.BorderLayout;
import java.awt.event.ActionEvent;

import javax.swing.AbstractAction;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.SwingUtilities;

/**
 * <code>ExMediaSdkPlayRealtimeVideoOnlyRtmpStream</code> - {@link IMediaSdk} example that plays real-time video-only RTMP stream.
 * <p> Note:
 * 
 * @author Andrei Sochirca
 * @see <a href="http://www.smaxe.com/product.jsf?id=juv-media-sdk" target="_blank">JUV Media SDK</a>
 * @see <a href="http://www.smaxe.com/product.jsf?id=juv-rtmp-client" target="_blank">JUV RTMP Client</a>
 */
public final class ExMediaSdkPlayRealtimeVideoOnlyRtmpStream extends Object
{
    /**
     * Entry point.
     * 
     * @param args
     * @throws Exception
     */
    public static void main(final String[] args) throws Exception
    {
        // NOTE:
        // Evaluation version has some limitations:
        // - video decoder watermarks frames (and blacks it sometimes)
        
        final String url = args.length == 0 ? "rtmp://184.72.239.149/vod/mp4:BigBuckBunny_175k.mov" : args[0];
        
        final IMediaSdk mediaSdk = MediaSdkFactory.getMediaSdk();
        
        final JVideoScreen videoScreen = new JVideoScreen();
        
        final JButton start = new JButton(new AbstractAction("Play")
        {
            private final static long serialVersionUID = -3538347901572402162L;
            
            public void actionPerformed(final ActionEvent e)
            {
                new Thread(new Runnable()
                {
                    public void run()
                    {
                        final RtmpPlayer player = new RtmpPlayer();
                        
                        player.setLogger(Loggers.createSoLogger(ILogger.INFO, "RtmpPlayer"));
                        
                        player.play(url, url.substring(url.lastIndexOf('/') + 1), new FlvVideo(new MediaStreamAcceptor(mediaSdk, videoScreen)));
                    }
                }, "ExMediaSdkPlayRealtimeVideoOnlyRtmpStream").start();
            }
        });
        
        final JFrame frame = new JFrame();
        
        frame.getContentPane().setLayout(new BorderLayout());
        frame.getContentPane().add(videoScreen, BorderLayout.CENTER);
        frame.getContentPane().add(start, BorderLayout.SOUTH);
        frame.pack();
        
        frame.setTitle(url);
        frame.setVisible(true);
    }
    
    /**
     * <code>MediaStreamAcceptor</code>
     */
    private final static class MediaStreamAcceptor extends Object implements IMediaStreamAcceptor
    {
        // fields
        private final IMediaSdk mediaSdk;
        private final JVideoScreen videoScreen;
        
        // state
        private IVideoDecoder videoDecoder;
        private MediaTrackInfo videoTrack;
        
        /**
         * Constructor.
         * 
         * @param mediaSdk
         * @param videoScreen
         */
        public MediaStreamAcceptor(final IMediaSdk mediaSdk, final JVideoScreen videoScreen)
        {
            this.mediaSdk = mediaSdk;
            this.videoScreen = videoScreen;
        }
        
        // IMediaStreamAcceptor implementation
        
        public void onAudioData(final MediaData data)
        {
        }
        
        public void onVideoData(final MediaData data)
        {
            final ByteArray payload = MediaDataFactory.getMediaDataPayload(data);
            final int videoCodec = MediaDataFactory.getVideoCodec(payload);
            
            if (MediaDataFactory.isConfigurationPayload(videoCodec, payload))
            {
                final MediaTrackInfo track = MediaDataFactory.createMediaTrackInfo(videoCodec, payload);
                
                if (!track.equals(videoTrack))
                {
                    videoTrack = track;
                    
                    videoDecoder = mediaSdk.getMediaCodecSdk().createVideoDecoder(videoTrack.codec, videoTrack.codecConfig, null);
                }
            }
            
            if (!MediaDataFactory.isFramePayload(videoCodec, payload)) return;
            
            final ByteArray encodedFrame = MediaDataFactory.extractEncodedFrame(videoCodec, payload);
            
            videoDecoder.decodeFrame(data.timestamp, encodedFrame, new IVideoDecoder.CallbackAdapter()
            {
                @Override
                public void onDecodedFrame(final VideoFrame frame)
                {
                    SwingUtilities.invokeLater(new Runnable()
                    {
                        public void run()
                        {
                            videoScreen.setFrame(frame);
                        }
                    });
                }
                
                @Override
                public void onException(final Object tag, final Exception e)
                {
                    e.printStackTrace();
                }
            });
        }
        
        public void onNotify(final Object data)
        {
        }
        
        public void release()
        {
        }
    }
    
    /**
     * Constructor.
     */
    private ExMediaSdkPlayRealtimeVideoOnlyRtmpStream()
    {
    }
}