/**
 * Copyright (c) 2008 - 2014 Smaxe Ltd (www.smaxe.com).
 * All rights reserved.
 */

import com.smaxe.logger.ILogger;
import com.smaxe.logger.support.Loggers;
import com.smaxe.uv.client.rtsp.IVideo;
import com.smaxe.uv.client.rtsp.video.AbstractVideo;
import com.smaxe.uv.client.rtsp.video.DispatcherVideo;
import com.smaxe.uv.client.x.RtspPlayer;
import com.smaxe.uv.media.core.IVideoDecoder;
import com.smaxe.uv.media.core.MediaTrackInfo;
import com.smaxe.uv.media.core.VideoFrame;
import com.smaxe.uv.media.java.swing.JVideoScreen;
import com.smaxe.uv.media.sdk.IMediaSdk;
import com.smaxe.uv.media.sdk.MediaSdkFactory;
import com.smaxe.uv.stream.MediaData;
import com.smaxe.uv.stream.MediaDataFactory;

import java.awt.BorderLayout;
import java.awt.event.ActionEvent;

import javax.swing.AbstractAction;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.SwingUtilities;

/**
 * <code>ExMediaSdkPlayRealtimeVideoOnlyRtspStream</code> - {@link IMediaSdk} example
 * that plays real-time video-only RTSP/RTP stream.
 * <p> Note:
 * 
 * @author Andrei Sochirca
 * @see <a href="http://www.smaxe.com/product.jsf?id=juv-media-sdk" target="_blank">JUV Media SDK</a>
 * @see <a href="http://www.smaxe.com/product.jsf?id=juv-rtsp-client" target="_blank">JUV RTSP/RTP Client</a>
 */
public final class ExMediaSdkPlayRealtimeVideoOnlyRtspStream extends Object
{
    /**
     * Entry point.
     * 
     * @param args
     * @throws Exception
     */
    public static void main(final String[] args) throws Exception
    {
        // NOTE:
        // Evaluation version has some limitations:
        // - video decoder watermarks frames (and blacks it sometimes)
        
        final String url = args.length == 0 ? "rtsp://184.72.239.149/vod/mp4:BigBuckBunny_175k.mov" : args[0];
        
        final IMediaSdk mediaSdk = MediaSdkFactory.getMediaSdk();
        
        final JVideoScreen videoScreen = new JVideoScreen();
        
        videoScreen.scale(0.5f);
        
        final JButton start = new JButton(new AbstractAction("Play")
        {
            private final static long serialVersionUID = -3538347901572402162L;
            
            public void actionPerformed(final ActionEvent e)
            {
                new Thread(new Runnable()
                {
                    public void run()
                    {
                        final RtspPlayer player = new RtspPlayer();
                        
                        player.setLogger(Loggers.createSoLogger(ILogger.INFO, "RtspPlayer"));
                        
                        player.play(url, new DispatcherVideo(new Video(mediaSdk, videoScreen)));
                    }
                }, "ExMediaSdkPlayRealtimeVideoOnlyRtspStream").start();
            }
        });
        
        final JFrame frame = new JFrame();
        
        frame.getContentPane().setLayout(new BorderLayout());
        frame.getContentPane().add(videoScreen, BorderLayout.CENTER);
        frame.getContentPane().add(start, BorderLayout.SOUTH);
        frame.pack();
        
        frame.setTitle(url);
        frame.setVisible(true);
    }
    
    /**
     * <code>Video</code>
     */
    private final static class Video extends AbstractVideo implements IVideo
    {
        // fields
        private final IMediaSdk mediaSdk;
        private final JVideoScreen videoScreen;
        
        // state
        private IVideoDecoder videoDecoder;
        
        /**
         * Constructor.
         * 
         * @param mediaSdk
         * @param videoScreen
         */
        public Video(final IMediaSdk mediaSdk, final JVideoScreen videoScreen)
        {
            this.mediaSdk = mediaSdk;
            this.videoScreen = videoScreen;
        }
        
        // IVideo implementation
        
        @Override
        public void onMediaData(final MediaTrackInfo track, final MediaData data)
        {
            System.out.println("onMediaData: " + track + "  " + data);
            
            if (track.type != MediaTrackInfo.VIDEO) return;
            
            if (videoDecoder == null)
            {
                videoDecoder = mediaSdk.getMediaCodecSdk().createVideoDecoder(track.codec, track.codecConfig, null);
            }
            
            videoDecoder.decodeFrame(data.timestamp, MediaDataFactory.getMediaDataPayload(data), new IVideoDecoder.CallbackAdapter()
            {
                @Override
                public void onDecodedFrame(final VideoFrame frame)
                {
                    SwingUtilities.invokeLater(new Runnable()
                    {
                        public void run()
                        {
                            videoScreen.setFrame(frame);
                        }
                    });
                }
                
                @Override
                public void onException(final Object tag, final Exception e)
                {
                    e.printStackTrace();
                }
            });
        }
    }
    
    /**
     * Constructor.
     */
    private ExMediaSdkPlayRealtimeVideoOnlyRtspStream()
    {
    }
}